#!/usr/bin/env python3
"""
A Simple Python Terminal Window Example
by Javantea
Jan 31, 2015
"""
from __future__ import print_function
from gi.repository import Gtk, GObject, Vte

# Use glade for just at window. pyterm1.glade isn't anything but a window 
# called termwin. Why? Because glade is actually very useful when you get more 
# complex.
builder = Gtk.Builder()
builder.add_from_file('pyterm1.glade')
window = builder.get_object("termwin")
box1 = builder.get_object("box1")

# Instead of using the Vte glade setup, I do it in here because I forgot to 
# compile vte with glade support.
t = Vte.Terminal.new()
pty = t.pty_new_sync(Vte.PtyFlags.NO_UTMP | Vte.PtyFlags.NO_LASTLOG | 
	Vte.PtyFlags.NO_WTMP, None)

def prompt(term):
	term.feed(b">>> ")
#end def prompt(term)

prompt(t)

def gtkExit(widget=None):
	"""
	Useful quit function.
	"""
	#print("Quit was called", widget)
	pty.close()
	Gtk.main_quit(widget)
#end def gtkExit(widget)

command_data = ''

def commit(widget, data, length):
	"""
	An interesting function that takes in strings from the virtual 
	terminal. When you type it adds to command_data. When you 
	backspace, it removes a character from command_data. When you 
	type Ctrl-D, it quits. When you type newline, it evals the 
	command_data string. Careful this is intentional code execution. 
	Unlike a real Python shell it doesn't have normal left-hand 
	right-hand syntax like a = 3. Instead it only accepts right-hand 
	syntax because of eval. It doesn't allow multiple lines and it 
	doesn't print your exceptions on the terminal, it outputs them to 
	stdout instead (which is probably a good thing).
	
	"""
	global command_data
	#print("Commit", repr(data), length)
	# Newline is screwy, not sure why.
	newline = False
	if data == '\r':
		newline = True
		data = '\r\n'
	elif data == '\x08':
		# Backspace
		command_data = command_data[:-1]
	elif data == '\x04':
		# Ctrl-D exits.
		gtkExit(widget)
	else:
		command_data += data
	#end if
	# Make it visible
	widget.feed(data.encode('utf-8'))
	if newline:
		# Eval and print the result.
		print('eval:', repr(command_data))
		try:
			eval(command_data)
		except Exception as e:
			# TODO: Put it in the command window.
			prompt(widget)
			command_data = ''
			raise e
			return
		#end try
		prompt(widget)
		command_data = ''
	#end if
#end def commit(widget, data, length)

t.connect('commit', commit)

handlers = {"on_termwin_destroy": gtkExit}
builder.connect_signals(handlers)
box1.add(t)
# Terminal is a the top. The rest is below.
box1.reorder_child(t, 0)
window.show_all()
Gtk.main()
# It works!
